// This is the main DLL file.

#include "stdafx.h"

#include "G35DDCOOAPIWrapper.h"

using namespace G35DDCOOAPIWrapperSpace;
using namespace System;
using namespace System::Runtime::InteropServices;


//
// Notification
//
void __stdcall G35DDCOOAPIWrapperNotification::RegisterCallbacks(DDC2_PREPROCESSED_STREAM_CALLBACK theDDC2PreprocessedStreamCallback, AUDIO_STREAM_CALLBACK theAudioStreamCallback) // Callbacks from unmanaged to managed C++
{
	ddc2PreprocessedStreamCallback = theDDC2PreprocessedStreamCallback;
	audioStreamCallback = theAudioStreamCallback;
}

void __stdcall G35DDCOOAPIWrapperNotification::G35DDC_IFCallback(IG35DDCDevice* theDevice, const SHORT* theBuffer, UINT32 theNumberOfSamples, WORD theMaxAdcAmplitude, UINT32 theADCSamplingRate)
{
	// Not used in this example
}

void __stdcall G35DDCOOAPIWrapperNotification::G35DDC_DDC1StreamCallback(IG35DDCDevice* theDevice, const VOID* theBuffer, UINT32 theNumberOfSamples, UINT32 theBitsPerSample)
{
	// Not used in this example
}

BOOL __stdcall G35DDCOOAPIWrapperNotification::G35DDC_DDC1PlaybackStreamCallback(IG35DDCDevice* theDevice, VOID* theBuffer, UINT32 theNumberOfSamples, UINT32 theBitsPerSample)
{
	return FALSE; // Not used in this example
}

void __stdcall G35DDCOOAPIWrapperNotification::G35DDC_DDC2StreamCallback(IG35DDCDevice* theDevice, UINT32 theChannel, CONST FLOAT* theBuffer, UINT32 theNumberOfSamples)
{
	// Not used in this example
}

void __stdcall G35DDCOOAPIWrapperNotification::G35DDC_DDC2PreprocessedStreamCallback(IG35DDCDevice* theDevice, UINT32 theChannel, CONST FLOAT* theBuffer, UINT32 theNumberOfSamples, FLOAT theSLevelPeak, FLOAT theSLevelRms)
{
	ddc2PreprocessedStreamCallback(theDevice, theChannel, theBuffer, theNumberOfSamples, theSLevelPeak, theSLevelRms); // Callback to managed C++
}

void __stdcall G35DDCOOAPIWrapperNotification::G35DDC_AudioStreamCallback(IG35DDCDevice* theDevice, UINT32 theChannel, UINT32 theType, CONST FLOAT* theBuffer, UINT32 theNumberOfSamples)
{
	audioStreamCallback(theDevice, theChannel, theType, theBuffer, theNumberOfSamples); // Callback to managed C++
}

BOOL __stdcall G35DDCOOAPIWrapperNotification::G35DDC_AudioPlaybackStreamCallback(IG35DDCDevice* theDevice, UINT32 theChannel, FLOAT* theBuffer, UINT32 theNumberOfSamples)
{
	return FALSE; // Not used in this example
}


//
// ApiHandler
//
HMODULE ourModuleHandle = NULL;
G3XDDCAPI_CREATE_INSTANCE ourCreateInstance;
IG35DDCDevice* ourDevice = NULL;
G35DDCOOAPIWrapperNotification ourNotification;

G35DDCOOAPIWrapper::G35DDCOOAPIWrapper()
{
	instance = this; // Used in callback from managed C++ to C#
	devicesCount = 0;

	IntPtr aPointer = Marshal::StringToHGlobalUni(APILIBRARY);
	ourModuleHandle = LoadLibrary((LPCWSTR)aPointer.ToPointer());

	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("Failed to load {0}", APILIBRARY);
		status = STATUS_APILIBRARY_NOT_FOUND;
		return;
	}

    // Retrieve address of CreateInstance API function
    ourCreateInstance = (G3XDDCAPI_CREATE_INSTANCE)GetProcAddress(ourModuleHandle, "CreateInstance");

	if (!ourCreateInstance)
	{
		Console::WriteLine("Incorrect {0}", APILIBRARY);
		FreeLibrary(ourModuleHandle);
		ourModuleHandle = NULL;
		status = STATUS_APILIBRARY_INCORRECT;
		return;
	}

	Console::WriteLine("{0} loaded", APILIBRARY);

	status = STATUS_APILIBRARY_LOADED;
}

G35DDCOOAPIWrapper::~G35DDCOOAPIWrapper()
{
	FreeLibrary(ourModuleHandle);
}

DWORD G35DDCOOAPIWrapper::GetStatus()
{
	return status;
}

BOOL G35DDCOOAPIWrapper::IsConnected()
{
	if (ourDevice == NULL)
	{
		return FALSE;
	}
	else
	{
		return ourDevice->IsConnected();
	}
}

String^ G35DDCOOAPIWrapper::GetDevicePath()
{
	return deviceInfo.devicePath;
}

BYTE G35DDCOOAPIWrapper::GetInterfaceType()
{
	return deviceInfo.interfaceType;
}

String^ G35DDCOOAPIWrapper::GetSerialNumber()
{
	return deviceInfo.serialNumber;
}

BOOL G35DDCOOAPIWrapper::Enumerate()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);
		return FALSE;
	}

	IG35DDCDeviceEnumerator* aDeviceEnumerator;
	G35DDC_DEVICE_INFO aDeviceInfo;

	BOOL aResult = FALSE;

	if (ourCreateInstance(G35DDC_CLASS_ID_DEVICE_ENUMERATOR, (void**)&aDeviceEnumerator))
	{
		aDeviceEnumerator->Enumerate();
		devicesCount = aDeviceEnumerator->GetCount();
        if (devicesCount > 0)
		{
			// Let's work with one device
			aDeviceEnumerator->GetDeviceInfo(0, &aDeviceInfo, sizeof(G35DDC_DEVICE_INFO));

			deviceInfo.devicePath = gcnew String(aDeviceInfo.DevicePath);
			deviceInfo.interfaceType = aDeviceInfo.InterfaceType;
			deviceInfo.serialNumber = gcnew String(aDeviceInfo.SerialNumber);
			deviceInfo.channelCount = aDeviceInfo.ChannelCount;
			deviceInfo.ddcTypeCount = aDeviceInfo.DDCTypeCount;

			Console::WriteLine("Device #{0} found", deviceInfo.serialNumber);

			aResult = TRUE;
		}
		else
		{
			Console::WriteLine("No available device found");
		}

		aDeviceEnumerator->Release();
	}
    else
    {
        Console::WriteLine("Failed to create device enumerator object");
    }

	return aResult;
}

BOOL G35DDCOOAPIWrapper::Open()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);
		return FALSE;
	}

	if (devicesCount == 0)
	{
		Console::WriteLine("No available device found");
		return FALSE;
	}

    // Create instance of device object
    if (!ourCreateInstance(G35DDC_CLASS_ID_DEVICE,(void**)&ourDevice))
    {
		Console::WriteLine("Failed to create device object. Error code = {0}", GetLastError());
        return FALSE;
    }

	IntPtr aPointer = Marshal::StringToHGlobalAnsi(deviceInfo.devicePath);
	const char* aDevicePath = static_cast<char*>(aPointer.ToPointer());
    // Open device
    if (!ourDevice->Open(aDevicePath))
    {
        Console::WriteLine("Unable to open device");
        ourDevice->Release();
        FreeLibrary(ourModuleHandle);
        return FALSE;
    }
	Marshal::FreeHGlobal(aPointer);

	Console::WriteLine("Device opened");

	// Set power
    ourDevice->SetPower(TRUE);

	// Example initialization
    ourDevice->SetDDC1(1); // Set DDC type of DDC1 to 1 (DDC Bandwidth = 24000, SampleRate = 32 kHz, BitsPerSample = 32)
    ourDevice->SetAGCParams(0, 0.020, 0.500, -15.0);
    ourDevice->SetAGC(0, TRUE);
    ourDevice->SetDemodulatorMode(0, G3XDDC_MODE_AM);
    ourDevice->SetDemodulatorFilterBandwidth(0, 8000);

	return TRUE;
}

BOOL G35DDCOOAPIWrapper::Close()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	// Unregister all callback functions
    ourDevice->SetCallback(NULL);

    ourDevice->SetPower(FALSE);

	ourDevice->Close();
	ourDevice->Release();

	ourDevice = NULL;

	Console::WriteLine("Device closed");

	return TRUE;
}

BOOL G35DDCOOAPIWrapper::SetFrequency(UINT32 theChannel, UINT32 theFrequency)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->SetFrequency(theChannel, theFrequency);
}

BOOL G35DDCOOAPIWrapper::GetFrequency(UINT32 theChannel, UINT32* theFrequency)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->GetFrequency(theChannel, theFrequency);
}


BOOL G35DDCOOAPIWrapper::SetPreamp(BOOL thePreamp)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->SetPreamp(thePreamp);
}

BOOL G35DDCOOAPIWrapper::GetPreamp(BOOL* thePreamp)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->GetPreamp(thePreamp);
}

BOOL G35DDCOOAPIWrapper::SetAttenuator(UINT32 theAttenuator)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->SetAttenuator(theAttenuator);
}

BOOL G35DDCOOAPIWrapper::GetAttenuator(UINT32* theAttenuator)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->GetAttenuator(theAttenuator);
}

BOOL G35DDCOOAPIWrapper::RegisterCallbacks(DDC2PreprocessedStreamCallback^ theDDC2PreprocessedStreamCallback, AudioStreamCallback^ theAudioStreamCallback)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	ddc2PreprocessedStreamCallback = theDDC2PreprocessedStreamCallback; // Callback to C#
	audioStreamCallback = theAudioStreamCallback; // Callback to C#
	ourDevice->SetCallback(&ourNotification);

	// DDC2PreprocessedStream-callback from unmanaged to managed C++
	NotificationDDC2PreprocessedStreamCallbackDelegate^ aDDC2PreprocessedStreamDelegate = gcnew NotificationDDC2PreprocessedStreamCallbackDelegate(NotificationDDC2PreprocessedStreamCallback);
	GCHandle aHandle = GCHandle::Alloc(aDDC2PreprocessedStreamDelegate);
	IntPtr aPointer = Marshal::GetFunctionPointerForDelegate(aDDC2PreprocessedStreamDelegate);
	DDC2_PREPROCESSED_STREAM_CALLBACK aDDC2PreprocessedStreamCallback = static_cast<DDC2_PREPROCESSED_STREAM_CALLBACK>(aPointer.ToPointer());

	// AudioStreamCallback from unmanaged to managed C++
	NotificationAudioStreamCallbackDelegate^ anAudioStreamCallbackDelegate = gcnew NotificationAudioStreamCallbackDelegate(NotificationAudioStreamCallback);
	aHandle = GCHandle::Alloc(anAudioStreamCallbackDelegate);
	aPointer = Marshal::GetFunctionPointerForDelegate(anAudioStreamCallbackDelegate);
	AUDIO_STREAM_CALLBACK anAudioStreamCallback = static_cast<AUDIO_STREAM_CALLBACK>(aPointer.ToPointer());

	// Force garbage collection cycle to prove that the delegate does not get disposed
	GC::Collect();

	// Register callbacks from unmanaged to managed C++
	ourNotification.RegisterCallbacks(aDDC2PreprocessedStreamCallback, anAudioStreamCallback);

	return TRUE;
}

BOOL G35DDCOOAPIWrapper::StartDDC1(UINT32 theSamplesPerBuffer)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StartDDC1(theSamplesPerBuffer);
}

BOOL G35DDCOOAPIWrapper::StopDDC1()
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StopDDC1();
}

BOOL G35DDCOOAPIWrapper::StartDDC2(UINT32 theChannel, UINT32 theSamplesPerBuffer)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StartDDC2(theChannel, theSamplesPerBuffer);
}

BOOL G35DDCOOAPIWrapper::StopDDC2(UINT32 theChannel)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StopDDC2(theChannel);
}

BOOL G35DDCOOAPIWrapper::StartAudio(UINT32 theChannel, UINT32 theSamplesPerBuffer)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StartAudio(theChannel, theSamplesPerBuffer);
}

BOOL G35DDCOOAPIWrapper::StopAudio(UINT32 theChannel)
{
	if (ourModuleHandle == NULL)
	{
		Console::WriteLine("{0} not loaded", APILIBRARY);

		return FALSE;
	}

	if (ourDevice == NULL)
	{
		Console::WriteLine("Device not opened");

		return FALSE;
	}

	return ourDevice->StopAudio(theChannel);
}

void G35DDCOOAPIWrapper::NotificationDDC2PreprocessedStreamCallback(IG35DDCDevice* theDevice, UINT32 theChannel, const FLOAT* theBuffer, UINT32 theNumberOfSamples, FLOAT theSLevelPeak, FLOAT theSLevelRMS)
{
	// Callback to C#
	instance->ddc2PreprocessedStreamCallback(theChannel, theBuffer, theNumberOfSamples, theSLevelPeak, theSLevelRMS);
}

void G35DDCOOAPIWrapper::NotificationAudioStreamCallback(IG35DDCDevice* theDevice, UINT32 theChannel, UINT32 theType, const FLOAT* theBuffer, UINT32 theNumberOfSamples)
{
	// Callback to C#
	instance->audioStreamCallback(theChannel, theType, theBuffer, theNumberOfSamples);
}